@@ -561,7 +561,7 @@ display_about_dialog (BonoboUIComponent *uic,
 			       authors,
 			       documenters,
 			       translator_credits,
-			       "gnome-workspace",
+			       "gnome-panel-workspace-switcher",
 			       "pager",
 			       "Pager");
 }
@@ -870,6 +870,8 @@ setup_dialog (GladeXML  *xml,
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_workspaces_spin),
 				   wnck_screen_get_workspace_count (pager->screen));
+ 	gtk_spin_button_set_increments (GTK_SPIN_BUTTON (pager->num_workspaces_spin),
+					0.2, 0.2);
 	g_signal_connect (G_OBJECT (pager->num_workspaces_spin), "value_changed",
 			  (GCallback) num_workspaces_value_changed, pager);
 	
@@ -932,7 +934,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	}
 
 	gtk_window_set_icon_name (GTK_WINDOW (pager->properties_dialog),
-	                          "gnome-workspace");
+	                          "gnome-panel-workspace-switcher");
 	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
 			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
