@@ -65,7 +65,7 @@ struct _PanelPlaceMenuItemPrivate {
 	GtkWidget   *menu;
 	PanelWidget *panel;
 
-	EggRecentViewGtk *recent_view;
+	GtkRecentManager *recent_manager;
 
 	GnomeVFSMonitorHandle *bookmarks_monitor;
 
@@ -105,13 +105,15 @@ activate_uri (GtkWidget *menuitem,
 
 	if (error != NULL) {
 		if (error->code != GNOME_URL_ERROR_CANCELLED) {
+			char *primary;
 			escaped = g_markup_escape_text (url, -1);
-			panel_error_dialog (screen, "cannot_show_url", TRUE,
-					    _("Could not open location '%s'"),
-					    "%s",
-					    escaped,
-					    error->message);
+			primary = g_strdup_printf (_("Could not open location '%s'"),
+						   escaped);
 			g_free (escaped);
+			panel_error_dialog (NULL, screen,
+					    "cannot_show_url", TRUE,
+					    primary, error->message);
+			g_free (primary);
 		}
 		g_error_free (error);
 	}
@@ -128,6 +130,7 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 	GtkWidget *item;
 	char      *path_freeme;
 	char      *full_path;
+	char      *uri;
 	char      *icon;
 	char      *name;
 	char      *comment;
@@ -170,14 +173,11 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 		return;
 	}
 
-	icon    = g_key_file_get_locale_string (key_file, "Desktop Entry",
-						"Icon", NULL, NULL);
-	comment = g_key_file_get_locale_string (key_file, "Desktop Entry",
-						"Comment", NULL, NULL);
+	icon    = panel_util_key_file_get_locale_string (key_file, "Icon");
+	comment = panel_util_key_file_get_locale_string (key_file, "Comment");
 
 	if (string_empty (force_name))
-		name = g_key_file_get_locale_string (key_file, "Desktop Entry",
-						     "Name", NULL, NULL);
+		name = panel_util_key_file_get_locale_string (key_file, "Name");
 	else
 		name = g_strdup (force_name);
 
@@ -195,7 +195,10 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 			       (GClosureNotify) g_free, 0);
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
-	setup_uri_drag (item, full_path, icon);
+
+	uri = gnome_vfs_get_uri_from_local_path (full_path);
+	setup_uri_drag (item, uri, icon);
+	g_free (uri);
 
 	g_key_file_free (key_file);
 
@@ -407,6 +410,16 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 				char *buffer;
 
 				buffer = gnome_vfs_get_local_path_from_uri (full_uri);
+				if (!buffer) {
+					g_free (tooltip);
+					g_free (full_uri);
+					if (bookmark->label)
+						g_free (bookmark->label);
+					gnome_vfs_uri_unref (bookmark->uri);
+					g_free (bookmark);
+					continue;
+				}
+
 				label = g_filename_display_basename (buffer);
 				g_free (buffer);
 			} else if (gnome_vfs_uri_is_local (bookmark->uri)) {
@@ -504,7 +517,7 @@ panel_place_menu_item_append_volumes (GtkWidget *menu,
 		 * make the menu items smaller... */
 		if (connected_volumes) {
 			title = _("Network Places");
-			icon = "gnome-fs-bookmark";
+			icon = "network-server";
 		} else {
 			title = _("Removable Media");
 			icon = "";
@@ -551,7 +564,7 @@ panel_place_menu_item_append_volumes (GtkWidget *menu,
 
 
 static GtkWidget *
-panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
+panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 {
 	GtkWidget *places_menu;
 	GtkWidget *item;
@@ -570,7 +583,12 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 
 	if (!gconf_client_get_bool (panel_gconf_get_client (),
 				    DESKTOP_IS_HOME_DIR_KEY,
-				    NULL))
+				    NULL)) {
+		char *uri;
+
+		uri = gnome_vfs_make_uri_from_input_with_dirs ("Desktop",
+							       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
+
 		panel_menu_items_append_place_item (
 				"gnome-fs-desktop",
 				/* Translators: Desktop is used here as in
@@ -578,10 +596,13 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 				 * environment). Do not keep "Desktop Folder|"
 				 * in the translation */
 				Q_("Desktop Folder|Desktop"),
-				_("Open the desktop as a folder"),
+				_("Open the contents of your desktop in a folder"),
 				places_menu,
 				G_CALLBACK (activate_uri),
-				"Desktop");
+				uri);
+
+		g_free (uri);
+	}
 
 	panel_place_menu_item_append_gtk_bookmarks (places_menu);
 	add_menu_separator (places_menu);
@@ -620,8 +641,8 @@ panel_place_menu_item_create_menu (EggRecentViewGtk **recent_view)
 					      "gnome-search-tool.desktop",
 					      NULL);
 
-	*recent_view = panel_recent_append_documents_menu (places_menu,
-							   *recent_view);
+	panel_recent_append_documents_menu (places_menu,
+					    place_item->priv->recent_manager);
 
 	return places_menu;
 }
@@ -635,7 +656,7 @@ panel_place_menu_item_recreate_menu (GtkWidget *widget)
 
 	if (place_item->priv->menu) {
 		gtk_widget_destroy (place_item->priv->menu);
-		place_item->priv->menu = panel_place_menu_item_create_menu (&place_item->priv->recent_view);
+		place_item->priv->menu = panel_place_menu_item_create_menu (place_item);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (place_item),
 					   place_item->priv->menu);
 		panel_applet_menu_set_recurse (GTK_MENU (place_item->priv->menu),
@@ -672,29 +693,22 @@ panel_place_menu_item_volume_changed (GnomeVFSVolumeMonitor *monitor,
 }
 
 static void
-panel_desktop_menu_item_append_menu (GtkWidget            *menu,
-				     PanelDesktopMenuItem *parent)
+panel_desktop_menu_item_append_menu (GtkWidget *menu,
+				     gpointer   data)
 {
-	gboolean  add_separator;
-	GList    *children;
+	PanelDesktopMenuItem *parent;
+	gboolean              add_separator;
+	GList                *children;
+	GList                *last;
 
-	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-appending"))
-		return;
-
-	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-appending", NULL);
+	parent = PANEL_DESKTOP_MENU_ITEM (data);
 
 	add_separator = FALSE;
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
+	last = g_list_last (children);
 
-	if (children != NULL) {
-		GList *child;
-
-		child = children;
-		while (child->next != NULL)
-			child = child->next;
-
-		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (child->data));
-	}
+	if (last != NULL)
+		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
 
 	g_list_free (children);
 
@@ -715,9 +729,12 @@ panel_desktop_menu_item_create_menu (PanelDesktopMenuItem *desktop_item)
 
 	desktop_menu = create_applications_menu ("settings.menu", NULL);
 
-	g_signal_connect (desktop_menu, "show",
-			  G_CALLBACK (panel_desktop_menu_item_append_menu),
-			  desktop_item);
+	g_object_set_data (G_OBJECT (desktop_menu),
+			   "panel-menu-append-callback",
+			   panel_desktop_menu_item_append_menu);
+	g_object_set_data (G_OBJECT (desktop_menu),
+			   "panel-menu-append-callback-data",
+			   desktop_item);
 
 	return desktop_menu;
 }
@@ -748,10 +765,6 @@ panel_place_menu_item_finalize (GObject *object)
 				 NAMES_DIR,
 				 NULL);
 
-	if (menuitem->priv->recent_view != NULL)
-		g_object_unref (menuitem->priv->recent_view);
-	menuitem->priv->recent_view = NULL;
-
 	if (menuitem->priv->bookmarks_monitor != NULL)
 		gnome_vfs_monitor_cancel (menuitem->priv->bookmarks_monitor);
 	menuitem->priv->bookmarks_monitor = NULL;
@@ -808,6 +821,8 @@ panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
 					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
 					    G_OBJECT (menuitem));
 
+	menuitem->priv->recent_manager = gtk_recent_manager_get_default ();
+
 	bookmarks_filename = g_build_filename (g_get_home_dir (),
 					       BOOKMARKS_FILENAME, NULL);
 	bookmarks_uri = gnome_vfs_get_uri_from_local_path (bookmarks_filename);
@@ -958,7 +973,7 @@ panel_place_menu_item_new (gboolean use_image)
 					       image);
 	}
 
-	menuitem->priv->menu = panel_place_menu_item_create_menu (&menuitem->priv->recent_view);
+	menuitem->priv->menu = panel_place_menu_item_create_menu (menuitem);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
 				   menuitem->priv->menu);
 
@@ -974,10 +989,7 @@ panel_desktop_menu_item_new (gboolean use_image,
 
 	menuitem = g_object_new (PANEL_TYPE_DESKTOP_MENU_ITEM, NULL);
 
-	/* Translators: Desktop is used here as in "Desktop Environment" (this
-	 * is not the Desktop folder). Do not keep "Desktop Environment|" in
-	 * the translation. */
-	accel_label = gtk_accel_label_new (Q_("Desktop Environment|Desktop"));
+	accel_label = gtk_accel_label_new (_("System"));
 	gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
 
 	gtk_container_add (GTK_CONTAINER (menuitem), accel_label);
@@ -989,7 +1001,7 @@ panel_desktop_menu_item_new (gboolean use_image,
 	if (use_image) {
 		GtkWidget *image;
 
-		image = gtk_image_new_from_icon_name ("gnome-fs-desktop",
+		image = gtk_image_new_from_icon_name ("computer",
 						      panel_menu_icon_get_size ());
 
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
@@ -1039,6 +1051,7 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 {
 	gboolean    separator_inserted;
 	GList      *children;
+	GList      *last;
 	GtkWidget  *item;
 	const char *translate;
 	char       *label;
@@ -1046,12 +1059,11 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 
 	separator_inserted = FALSE;
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
-
-	if (children != NULL) {
-		while (children->next != NULL)
-			children = children->next;
-		separator_inserted = GTK_IS_SEPARATOR (GTK_WIDGET (children->data));
+	last = g_list_last (children);
+	if (last != NULL) {
+		separator_inserted = GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
 	}
+	g_list_free (children);
 
 	if (panel_lock_screen_action_available ("lock")) {
 		item = panel_menu_items_create_action_item (PANEL_ACTION_LOCK);
@@ -1118,34 +1130,17 @@ void
 panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 				       const char *path)
 {
-	GnomeDesktopItem *item;
-	GError           *error;
+	GError *error;
 
 	error = NULL;
-	item = gnome_desktop_item_new_from_file (path, 0, &error);
-	if (item) {
-		g_assert (error == NULL);
-
-		panel_ditem_launch (item, NULL, 0,
-				    menuitem_to_screen (menuitem), &error);
-		if (error) {
-			panel_error_dialog (menuitem_to_screen (menuitem),
-					    "cannot_launch_entry", TRUE,
-					    _("Could not launch menu item"),
-					    "%s",
-					    error->message);
-
-			g_error_free (error);
-		}
-		gnome_desktop_item_unref (item);
-	} else {
-		g_assert (error != NULL);
-
-		panel_error_dialog (menuitem_to_screen (menuitem),
-				    "cannot_load_entry", TRUE,
-				    _("Could not load menu item"),
-				    "%s",
+	panel_launch_desktop_file (path, NULL,
+				   menuitem_to_screen (menuitem), &error);
+	if (error) {
+		panel_error_dialog (NULL, menuitem_to_screen (menuitem),
+				    "cannot_launch_entry", TRUE,
+				    _("Could not launch menu item"),
 				    error->message);
+
 		g_error_free (error);
 	}
 }
